#include "row.h"

#include <TLabel>
#include <QLabel>
#include <TLed>
#include <TTable>
#include <TPushButton>
#include <QCheckBox>
#include <QtDebug>
#include <QHBoxLayout>
#include <QSpacerItem>
#include <elettracolors.h>
#include <QTWatcher>
#include <configuration.h>

Row::Row(const QStringList &parts, QWidget *parent) : QWidget(parent)
{
    QHBoxLayout *hlo = new QHBoxLayout(this);
    QLabel *lName = new QLabel(this);
    mSection = parts.first();
    mVlvfeSource = parts.at(1);
    mPhshDev = parts.at(2);
    QFont bold(lName->font());
    bold.setBold(true);
    lName->setText(parts.first().toUpper());
    lName->setAlignment(Qt::AlignHCenter);
    bold.setPointSize(bold.pointSize() + 1);
    bold.setUnderline(true);
    lName->setFont(bold);
    bold.setPointSize(bold.pointSize() - 1); /* restore size */
    bold.setUnderline(false);
    hlo->addWidget(lName);

    QTWatcher *stateW = new QTWatcher(this);
    connect(stateW->qtangoComHandle(), SIGNAL(newData(TVariant)), this, SLOT(valveStateChanged(TVariant)));
    stateW->setSource(mVlvfeSource);


    ELabel *stateLabel = new ELabel(this);
    stateLabel->setObjectName("stateLabel");
    hlo->addWidget(stateLabel);

    ELabel *remoteLabel = new ELabel(this);
    remoteLabel->setObjectName("remoteLabel");
    hlo->addWidget(remoteLabel);

    QCheckBox *cbVlvEnable = new QCheckBox(this);
    cbVlvEnable->setChecked(true);
    cbVlvEnable->setObjectName("cbValveEnable");
    hlo->addWidget(cbVlvEnable);


    QFrame* vFrame = new QFrame;
    vFrame->setFrameShape(QFrame::VLine);
    hlo->addWidget(vFrame);

    QLabel *lEnable = new QLabel(this);
    lEnable->setAlignment(Qt::AlignRight);
    lEnable->setText("Enable");
    hlo->addWidget(lEnable);

    TLed *tlEnable = new TLed(this);
    tlEnable->setSource(mPhshDev + "/AbiGroup");
    hlo->addWidget(tlEnable);

    TLabel *tlBshState = new TLabel(this);
    tlBshState->setSource(mPhshDev + "/State");
    hlo->addWidget(tlBshState);

    QCheckBox *cbEnable = new QCheckBox(this);
    cbEnable->setChecked(true);
    cbEnable->setObjectName("cbShutterEnable");
    hlo->addWidget(cbEnable);

    foreach(QCheckBox *cb, findChildren<QCheckBox *>())
        connect(cb, SIGNAL(toggled(bool)), this, SLOT(checkboxStateChanged(bool)));

}

bool Row::isValveSelected() const
{
    return findChild<QCheckBox *>("cbValveEnable")->isChecked();
}

bool Row::isShutterSelected() const
{
    return findChild<QCheckBox *>("cbShutterEnable")->isChecked();
}

QString Row::section() const
{
    return mSection;
}

QString Row::vlvFeSource() const
{
    return mVlvfeSource;
}

void Row::setValveSelected(bool select)
{
    findChild<QCheckBox *>("cbValveEnable")->setChecked(select);
    emit selectionChanged(true, select);
}

void Row::setShutterSelected(bool select)
{
    findChild<QCheckBox *>("cbShutterEnable")->setChecked(select);
    emit selectionChanged(false, select);
}

void Row::checkboxStateChanged(bool checked)
{
    emit selectionChanged((sender()->objectName() == "cbValveEnable"), checked);
}

void Row::valveStateChanged(const TVariant &v)
{
    ELabel *stateLab = findChild<ELabel *>("stateLabel");
    ELabel *remoteLab = findChild<ELabel *>("remoteLabel");
    stateLab->setEnabled(v.quality() != ATTR_INVALID);
    remoteLab->setEnabled(v.quality() != ATTR_INVALID);

    QVector<bool> vb = v.toBoolVector();
//    qDebug() << __FUNCTION__ << mVlvfeSource << mSection << "vb 0 " << vb.at(0) << " vb 1 " << vb.at(1) << vb.at(2);

    if(v.quality() == Tango::ATTR_INVALID)
            stateLab->setText("####");
    else if(v.quality() != Tango::ATTR_INVALID && vb.at(0) && !vb.at(1))
        stateLab->setText("OPEN");
    else if(v.quality() != Tango::ATTR_INVALID && !vb.at(0) && vb.at(1))
        stateLab->setText("CLOSE");
    else
        stateLab->setText("----");

    stateLab->setToolTip(v.message());
    QPalette pa = stateLab->palette();
    if(stateLab->text() == "OPEN")
        pa.setColor(QPalette::Background, Config::instance()->stateColor(Tango::OPEN));
    else
        pa.setColor(QPalette::Background, Qt::white);
    stateLab->setPalette(pa);

    if(vb.size() > 2 && !vb.at(2))
        remoteLab->setText("REMOTE");
    else if(vb.size() > 2)
        remoteLab->setText("LOCAL");
    else
        remoteLab->setText("####");

}

bool Row::isVlvFETango() const
{
    return !mVlvfeSource.contains("bridge");
}

QString Row::shutterSource() const
{
    return mPhshDev;
}
