#include "rpcthread.h"
#include <macros.h>

#ifndef NO_ELETTRA_RPC
#include <lpc.h>
#include <RpcResolver.h>
#endif

RpcThread::RpcThread(QObject *parent, const QStringList &sections, const QString &operation) : QThread(parent)
{
    mSections = sections;
    mOperation = operation;
}

bool RpcThread::hasErrors() const
{
    return mErrors.size() > 0;
}

bool RpcThread::isCancelled() const
{
    return mCancelled;
}

QStringList RpcThread::errors() const
{
    return mErrors;
}

void RpcThread::cancel()
{
    mCancelled = true;
}

void RpcThread::run()
{
    char *host;
    short port;
    long sid = -1;
    int res;
    int total = mSections.size();
    int step = 0;
    mErrors.clear();
    mCancelled = false;

    foreach(QString section, mSections)
    {
        step++;
        if(!mCancelled)
        {

            sid = -1;

            /// TEST
#ifdef NO_ELETTRA_RPC
            QString noElettraRpcTest =  QString("#define NO_ELETTRA_RPC: dummy %1 operation on valve section \"%2\"").arg(mOperation).arg(section);
            mErrors << noElettraRpcTest;
            usleep(500000);
            /// end TEST
#else

            RpcResolver resolver;
            res = resolver.resolve("VLVFE", qstoc(section.toUpper()), qstoc(mOperation.toUpper()), "WP", &host, &port);
            if(res < 0)
            {
                mErrors << QString("Error resolving RPC server name and port for the section %1!").arg(section);
            }
            else
            {
                sid = import_lpc(host, port); /* obtain it.. */
                unsigned char data;
                long res = -1;
                res = lpc_WP("VLVFE", qstoc(section.toUpper()), qstoc(mOperation.toUpper()), "WP", &data);
                if(res < 0)
                    mErrors << QString("Error resolving RPC server name and port for the section %1").arg(section);
                else
                    printf("successfully executed lpc_WP VLVFE %s %s \"WP\"\n",  qstoc(section.toUpper()), qstoc(mOperation.toUpper()));
                free_lpc(sid);
            }
#endif
        }
        else
            step = total; /* end if cancelled */

        emit progress(step, total);

    }
}
